import DocumentLock from "./lock";
import { ITypedQuery, IQuery } from "./query";
import { ILoggingInterface } from "@hibas123/logging";
import Session from "./session";
import { Observable } from "@hibas123/utils";
import { IRuleEngine } from "../rules";
export declare type ChangeTypes = "added" | "modified" | "deleted";
export declare type Change = {
    data: any;
    document: string;
    collection: string;
    type: ChangeTypes;
    sender: string;
};
export declare class Database {
    #private;
    database_path: string;
    logging: ILoggingInterface;
    static getKey(collectionid: string, documentid?: string): string;
    get data(): import("../storage").LevelDB;
    get collections(): import("../storage").LevelDB;
    get rules(): IRuleEngine | undefined;
    set rules(engine: IRuleEngine | undefined);
    connectionCount: number;
    private locks;
    collectionLocks: DocumentLock;
    changeListener: Map<string, Set<(change: Change[]) => void>>;
    collectionChangeListener: Observable<{
        key: string;
        id: string;
        type: "create" | "delete";
    }>;
    toJSON(): {
        database_path: string;
        rules: IRuleEngine;
        connections: number;
    };
    constructor(database_path: string, ruleEngine?: IRuleEngine);
    resolve(path: string[], create?: boolean): Promise<{
        collection: string;
        document: string;
        collectionKey: string;
    }>;
    private sendChanges;
    private validate;
    run(queries: IQuery[], session: Session): Promise<any>;
    snapshot(rawQuery: ITypedQuery<"snapshot">, session: Session, onchange: (change: any) => void): Promise<{
        id: string;
        snaphot: any;
    }>;
    unsubscribe(id: string, session: Session): Promise<void>;
    stop(): Promise<void>;
    delete(): Promise<void>;
    runCleanup(): Promise<string[]>;
}
