"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRawLevelDB = exports.deleteRawLevelDB = exports.resNull = void 0;
const fs = require("fs");
const path = require("path");
const LUR = require("levelup");
const LDR = require("leveldown");
const LevelUp = LUR;
const LevelDown = LDR;
const databases = new Map();
function resNull(err) {
    if (!err.notFound)
        throw err;
    return null;
}
exports.resNull = resNull;
async function rmRecursice(path) {
    if (fs.existsSync(path)) {
        await Promise.all(fs.readdirSync(path).map(async (file) => {
            var curPath = path + "/" + file;
            if (fs.lstatSync(curPath).isDirectory()) {
                // recurse
                await rmRecursice(curPath);
            }
            else {
                // delete file
                await fs.promises.unlink(curPath);
            }
        }));
        await fs.promises.rmdir(path);
    }
}
async function deleteRawLevelDB(db_path) {
    db_path = path.resolve(db_path);
    if (!db_path || db_path === "")
        return;
    let db = databases.get(db_path);
    if (db) {
        if (db.data.isOpen())
            await db.data.close();
        if (db.collection.isOpen())
            await db.collection.close();
    }
    await rmRecursice(db_path);
}
exports.deleteRawLevelDB = deleteRawLevelDB;
function getRawLevelDB(db_path) {
    db_path = path.resolve(db_path);
    let db = databases.get(db_path);
    if (!db) {
        if (!fs.existsSync(db_path)) {
            fs.mkdirSync(db_path, { recursive: true });
        }
    }
    db = {
        data: db && db.data.isOpen()
            ? db.data
            : LevelUp(LevelDown(path.join(db_path + "/data"))),
        collection: db && db.collection.isOpen()
            ? db.collection
            : LevelUp(LevelDown(path.join(db_path + "/collection"))),
    };
    databases.set(db_path, db);
    return db;
}
exports.getRawLevelDB = getRawLevelDB;
//# sourceMappingURL=storage.js.map